dbus_client_libdir = File.expand_path('lib')
$LOAD_PATH.push(dbus_client_libdir)

require 'socket'
require 'base64'
require '../dbus'
require 'ed25519'
require 'json'
require 'time'
require 'openssl'

# databus_endpoint = "https://dbusapi.sigma-dev.cloudera.com"
@databus_access_key = ENV['DBUS_ACCESS_KEY']
@databus_secret_access_key = ENV['DBUS_SECRET_ACCESS_KEY']
@databus_stream_name = "MastodonClusterDeploymentLog"
@databus_partition_key = "Test-Log-Consumer"
@header = Dbus::Header.new({:'name' => 'host', :'value' => Socket.gethostname})

def submit_inline_record
  log = 'A log line.'
  payload = Base64.strict_encode64(log)
  record = Dbus::Record.new(
      {
          :'headers' => [@header],
          :'streamName' => @databus_stream_name,
          :'partitionKey' => @databus_partition_key,
          :'payload' => payload,
          :'payloadSize' => log.bytesize,
      })

  put_record_request = Dbus::PutRecordRequest.new(
      {
          :'record' => record
      })

  api_config = Dbus::Configuration.new
  api_config.scheme = "https"
  api_config.host = "dbusapi.sigma-dev.cloudera.com"
  api_config.debugging = true

  api_instance = Dbus::DefaultApi.new(Dbus::ApiClient.new(
      config = api_config,
      request_signer = method(:sign_request_ed25519)))
  api_instance.put_record(put_record_request)
end

def submit_large_record
  log = 'A log line.'
  record = Dbus::Record.new(
      {
          :'headers' => [@header],
          :'streamName' => @databus_stream_name,
          :'partitionKey' => @databus_partition_key,
          :'payloadSize' => 8 * 1024 * 1024,
      })

  put_record_request = Dbus::PutRecordRequest.new(
      {
          :'record' => record
      })

  api_config = Dbus::Configuration.new
  api_config.scheme = "https"
  api_config.host = "dbusapi.sigma-dev.cloudera.com"
  api_config.debugging = true

  api_instance = Dbus::DefaultApi.new(Dbus::ApiClient.new(
      config = api_config,
      request_signer = method(:sign_request_ed25519)))
  response = api_instance.put_record(put_record_request)
  puts response

  req_opts = {:method => :put, :'body' => log}
  s3_put_request = Typhoeus::Request.new(response.record.upload_url, req_opts)
  s3_put_response = s3_put_request.run
  s3_put_response.response_code
end

def sign_request_ed25519(http_method, content_type, path)
  auth_date = Time.now.utc.rfc822
  sign_method = 'ed25519v1'
  bytes_to_sign = "#{http_method.upcase}\n#{content_type}\n#{auth_date}\n#{path}\n#{sign_method}"
  seed = Base64.decode64(@databus_secret_access_key)
  signing_key = Ed25519::SigningKey.new(seed)
  signature = signing_key.sign(bytes_to_sign)
  auth_info = JSON[
      {
          :'access_key_id' => @databus_access_key,
          :'auth_method' => sign_method
      }]

  build_auth_headers(auth_date, auth_info, signature)
end

def sign_request_rsa(http_method, content_type, path)
  auth_date = Time.now.utc.rfc822
  sign_method = 'rsav1'
  bytes_to_sign = "#{http_method.upcase}\n#{content_type}\n#{auth_date}\n#{path}\n#{sign_method}"
  key = OpenSSL::PKey::RSA.new(@databus_secret_access_key)
  signature = key.sign(OpenSSL::Digest::SHA256.new, bytes_to_sign)
  auth_info = JSON[
      {
          :'access_key_id' => @databus_access_key,
          :'auth_method' => sign_method
      }]
  build_auth_headers(auth_date, auth_info, signature)
end

def build_auth_headers(auth_date, auth_info, signature)
  {
      :'x-altus-date' => auth_date,
      :'x-altus-auth' =>
          "#{Base64.urlsafe_encode64(auth_info)}.#{Base64.urlsafe_encode64(signature)}"
  }
end

submit_inline_record