# coding: utf-8
lib = File.expand_path('../lib', __FILE__)
$LOAD_PATH.unshift(lib) unless $LOAD_PATH.include?(lib)
require 'fluent/plugin/gcs/version'
require 'open3'



Gem::Specification.new do |spec|

  arg_file = File.expand_path("../GEMSPEC_ARGS", __FILE__)
  unless File.file?(arg_file)
    raise 'Missing GEMSPEC_ARGS, please build the gem by "make build".'
  end

  args = []
  File.open(arg_file).each_line do |line|
    line = line.chomp
    unless line.empty?
      args.push(line)
    end
  end

  version_pattern = '^[0-9]+\.[0-9]+\.[0-9]+(\.pre)?$'
  version = args[0]
  unless version.=~ /#{version_pattern}/
    raise StandardError, "Invalid version. Should be {number}.{number}.{number}(.pre)?"
  end

  latest_rel = Open3.capture2('git describe --tags --match "v[0-9]*.[0-9]*.[0-9]*" --always | cut -d "-" -f 1')[0].strip
  if latest_rel.=~ /v#{version_pattern}/ and version == latest_rel[1..latest_rel.length]
    raise StandardError, "Cannot build in released version: #{version}"
  end

  spec.name          = "fluent-plugin-gcs"
  spec.version       = version
  spec.authors       = ["Cloudera Inc."]
  spec.email         = ["no-repy@cloudera.com"]
  spec.summary       = "Google Cloud Storage output plugin for Fluentd"
  spec.description   = "Google Cloud Storage output plugin for Fluentd"
  spec.homepage      = "https://github.infra.cloudera.com/thunderhead/fluent-plugin-gcs"
  spec.license       = "Apache-2.0"

  spec.files         = `git ls-files -z`.split("\x0").reject do |f|
    f.match(%r{^(test|spec|features)/})
  end
  spec.bindir        = "exe"
  spec.executables   = spec.files.grep(%r{^exe/}) { |f| File.basename(f) }
  spec.require_paths = ["lib"]

  spec.add_runtime_dependency "fluentd", [">= 0.14.22", "< 2"]
  spec.add_runtime_dependency "google-cloud-storage", "~> 1.1"

  spec.add_development_dependency "rake", "~> 10.0"
  spec.add_development_dependency "rr", "= 1.1.2"
  spec.add_development_dependency "test-unit", ">= 3.0.8"
  spec.add_development_dependency "test-unit-rr", ">= 1.0.3"
  spec.add_development_dependency "timecop"
end
